#include "include/common_includes.h"
#include "kernel.h"
#define internal 
#include "idt_gdt_isr.h"
#define irq0_int 0x20
#define irq8_int 0x28
//#define enable_proceedings
#define idt_len 256
#define gdt_len 5
gdt_entry_t GDT[gdt_len];
gdt_ptr_t gdt_ptr;
idt_entry_t IDT[idt_len];
idt_ptr_t idt_ptr;

isr_t ihandlers[16];
int gpfs=0;
static void init_gdt()
{
	
	
	gdt_ptr.limit=(sizeof(gdt_entry_t)*gdt_len)-1;
	gdt_ptr.base=(uint32)&GDT;
	#ifdef enable_proceedings
	mprintf(newline " Initiating GDT",special_text_colour);
	mprintf(newline htab " setting 0-4 gates of GDT",screen_bcg_colour);
	#endif
	gdt_set_gate(0, 0, 0, 0, 0);
	gdt_set_gate(1, 0, 0xFFFFFFFF, 0x9A, 0xCF); // Code segment
	gdt_set_gate(2, 0, 0xFFFFFFFF, 0x92, 0xCF); // Data segment
	gdt_set_gate(3, 0, 0xFFFFFFFF, 0xFA, 0xCF); // User mode code segment
	gdt_set_gate(4, 0, 0xFFFFFFFF, 0xF2, 0xCF); // User mode data segment
	gdt_set_2((uint32)&gdt_ptr);
	
	#ifdef enable_proceedings
	mprintf(newline htab " successfully set 0-4 gates of GDT",screen_bcg_colour);
	mprintf(newline " GDT initialised : 5 entries",screen_bcg_colour);
	#endif
}

static void gdt_set_gate(int pos,uint32 base,uint32 limit,uint8 access,uint8 _granularity)
{
	GDT[pos].base_low=(base&0xffff);  //first 16 bits
	GDT[pos].base_middle=(base>>16)&0xff; //next 8 bits
	GDT[pos].base_high=(base>>24)&0xff;//last 8 bits
	
	GDT[pos].limit_low=(limit&0xffff);
	GDT[pos].granularity=(limit>>16)&0x0f;
	GDT[pos].granularity|= _granularity&0xf0;
	GDT[pos].access=access;		
}



static void init_idt()
{
	#ifdef enable_proceedings
	mprintf(newline " Initiating IDT",special_text_colour);
	#endif
	idt_ptr.limit=(sizeof(idt_entry_t)*idt_len)-1;
	idt_ptr.base=(uint32)&IDT;
	
	memset(&IDT,0,sizeof(idt_entry_t)*idt_len);
	#ifdef enable_proceedings
	mprintf(newline htab " setting 0-31 gates of IDT",screen_bcg_colour);
	#endif
	idt_set_gate( 0, (uint32)_isr0 , 0x08, 0x8E);
	idt_set_gate( 1, (uint32)_isr1 , 0x08, 0x8E);
	idt_set_gate( 2, (uint32)_isr2 , 0x08, 0x8E);
	idt_set_gate( 3, (uint32)_isr3 , 0x08, 0x8E);
	idt_set_gate( 4, (uint32)_isr4 , 0x08, 0x8E);
	idt_set_gate( 5, (uint32)_isr5 , 0x08, 0x8E);
	idt_set_gate( 6, (uint32)_isr6 , 0x08, 0x8E);
	idt_set_gate( 7, (uint32)_isr7 , 0x08, 0x8E);
	idt_set_gate( 8, (uint32)_isr8 , 0x08, 0x8E);
	idt_set_gate( 9, (uint32)_isr9 , 0x08, 0x8E);
	idt_set_gate( 10, (uint32)_isr10 , 0x08, 0x8E);
	idt_set_gate( 11, (uint32)_isr11 , 0x08, 0x8E);
	idt_set_gate( 12, (uint32)_isr12 , 0x08, 0x8E);
	idt_set_gate( 13, (uint32)_isr13 , 0x08, 0x8E);
	idt_set_gate( 14, (uint32)_isr14 , 0x08, 0x8E);
	idt_set_gate( 15, (uint32)_isr15 , 0x08, 0x8E);
	idt_set_gate( 16, (uint32)_isr16 , 0x08, 0x8E);
	idt_set_gate( 17, (uint32)_isr17 , 0x08, 0x8E);
	idt_set_gate( 18, (uint32)_isr18 , 0x08, 0x8E);
	idt_set_gate( 19, (uint32)_isr19 , 0x08, 0x8E);
	idt_set_gate( 20, (uint32)_isr20 , 0x08, 0x8E);
	idt_set_gate( 21, (uint32)_isr21 , 0x08, 0x8E);
	idt_set_gate( 22, (uint32)_isr22 , 0x08, 0x8E);
	idt_set_gate( 23, (uint32)_isr23 , 0x08, 0x8E);
	idt_set_gate( 24, (uint32)_isr24 , 0x08, 0x8E);
	idt_set_gate( 25, (uint32)_isr25 , 0x08, 0x8E);
	idt_set_gate( 26, (uint32)_isr26 , 0x08, 0x8E);
	idt_set_gate( 27, (uint32)_isr27 , 0x08, 0x8E);
	idt_set_gate( 28, (uint32)_isr28 , 0x08, 0x8E);
	idt_set_gate( 29, (uint32)_isr29 , 0x08, 0x8E);
	idt_set_gate( 30, (uint32)_isr30 , 0x08, 0x8E);
	idt_set_gate( 31, (uint32)_isr31 , 0x08, 0x8E);
	#ifdef enable_proceedings
    mprintf(newline htab " successfull in setting 0-31 gates of IDT",screen_bcg_colour);
		
	mprintf(newline htab " setting 32-47 gates of IDT",screen_bcg_colour);
	#endif
	//idt_set_gate( 32, (uint32)irq_0 , 0x08, 0x8E);
	idt_set_gate( 33, (uint32)irq_1 , 0x08, 0x8E);
	idt_set_gate( 34, (uint32)irq_2 , 0x08, 0x8E);
	idt_set_gate( 35, (uint32)irq_3 , 0x08, 0x8E);
	idt_set_gate( 36, (uint32)irq_4 , 0x08, 0x8E);
	idt_set_gate( 37, (uint32)irq_5 , 0x08, 0x8E);
	idt_set_gate( 38, (uint32)irq_6 , 0x08, 0x8E);
	idt_set_gate( 39, (uint32)irq_7 , 0x08, 0x8E);
	idt_set_gate( 40, (uint32)irq_8 , 0x08, 0x8E);
	idt_set_gate( 41, (uint32)irq_9 , 0x08, 0x8E);
	idt_set_gate( 42, (uint32)irq_10, 0x08, 0x8E);
	idt_set_gate( 43, (uint32)irq_11, 0x08, 0x8E);
	idt_set_gate( 44, (uint32)irq_12, 0x08, 0x8E);
	idt_set_gate( 45, (uint32)irq_13, 0x08, 0x8E);
	idt_set_gate( 46, (uint32)irq_14, 0x08, 0x8E);
	idt_set_gate( 47, (uint32)irq_15, 0x08, 0x8E);
	
	#ifdef enable_proceedings
	mprintf(newline htab " successfull in setting 32-47 gates of IDT",screen_bcg_colour);	
	mprintf(newline htab " Remapping IRQ 0-15 from 0x0-0xF to 0x20-0x2F",screen_bcg_colour);
	#endif
	
	remap_irqs();
	idt_set_2((uint32)&idt_ptr);
	
	#ifdef enable_proceedings
	mprintf(newline htab " IRQ remapping successful",screen_bcg_colour);	
	mprintf(newline htab " 47 entries done",screen_bcg_colour);
	mprintf(newline htab " 209 null entries done",screen_bcg_colour);
    mprintf(newline " IDT initialised :256 entries done",screen_bcg_colour);
	#endif
}

static void idt_set_gate(uint8 pos,uint32 base,uint16 sel,uint8 flags)
{
	IDT[pos].base_low=base&0xffff;
	IDT[pos].base_high=(base>>16)&0xffff;
	
	IDT[pos].selector=sel;
	IDT[pos].zero=0;
	
	IDT[pos].flags=flags;
}

void remap_irqs()
{
	outportb(0x20, 0x11);
	outportb(0xA0, 0x11);
	outportb(0x21, irq0_int);
	outportb(0xA1, irq8_int);
	outportb(0x21, 0x04);
	outportb(0xA1, 0x02);
	outportb(0x21, 0x01);
	outportb(0xA1, 0x01);
	//outportb(0x21, ~0x03);
	//outportb(0xA1, ~0x00);
}


void interrupt_handler(regs_t regs)
{
	if(ihandlers[regs.int_no]!=0)
	{
		isr_t ihandler=ihandlers[regs.int_no];
		ihandler(regs);
		//print_regs(regs);
	}
	else
	{
		mprintf(" IRQ ",screen_bcg_colour);
		print_long_(regs.int_no); //mprintf(newline,screen_bcg_colour);
		//print_regs(regs);		
	}
	send_EOI(regs.int_no);	
}

void print_regs(regs_t r)
{
	mprintf(newline,screen_bcg_colour);
	print_long_(r.ds);mprintf(", ",screen_bcg_colour);
	print_long_(r.edi);mprintf(", ",screen_bcg_colour);
	print_long_(r.esi);mprintf(", ",screen_bcg_colour);
	print_long_(r.ebp);mprintf(", ",screen_bcg_colour);
	print_long_(r.esp);mprintf(", ",screen_bcg_colour);
	print_long_(r.ebx);mprintf(", ",screen_bcg_colour);
	print_long_(r.edx);mprintf(", ",screen_bcg_colour);
	print_long_(r.ecx);mprintf(", ",screen_bcg_colour);
	print_long_(r.eax);mprintf(", ",screen_bcg_colour);
	mprintf(" int_no= ",screen_bcg_colour);
	print_long_(r.int_no);mprintf(", ",screen_bcg_colour);
	print_long_(r.err_code);mprintf(", ",screen_bcg_colour);
	print_long_(r.eip);mprintf(", ",screen_bcg_colour);
	print_long_(r.cs);mprintf(", ",screen_bcg_colour);
	print_long_(r.eflags);mprintf(", ",screen_bcg_colour);
	print_long_(r.useresp);mprintf(", ",screen_bcg_colour);
	print_long_(r.ss);mprintf(", ",screen_bcg_colour);
}
char *isr_faults[]={" Division By Zero",
    " Debug",
    " Non Maskable Interrupt",
    " Breakpoint",
    " Into Detected Overflow",
    " Out of Bounds",
    " Invalid Opcode",
    " No Coprocessor",

    " Double Fault",
    " Coprocessor Segment Overrun",
    " Bad TSS",
    " Segment Not Present",
    " Stack Fault",
    " General Protection Fault",
    " Page Fault",
    " Unknown Interrupt",

    " Coprocessor Fault",
    " Alignment Check",
    " Machine Check",
    " Reserved",
    " Reserved",
    " Reserved",
    " Reserved",
    " Reserved",

    " Reserved",
    " Reserved",
    " Reserved",
    " Reserved",
    " Reserved",
    " Reserved",
    " Reserved",
    " Reserved"};
    
uint32 fault=0;
void isr_handler(regs_t regs)
{
	uint16 pos;
	pos=get_cursor_pos()/2;	
	mprintf(newline,screen_bcg_colour);
	set_cursor_pos(0);
	print_long_(regs.unknown);
	mprintf(isr_faults[regs.unknown],screen_bcg_colour);
	print_long_(gpfs);
	set_cursor_pos(pos);
	//print_regs(regs);
	
	if(regs.int_no==13)
	  gpfs++;
	  
	if(regs.unknown){}
	send_EOI(regs.unknown);
		
	//while(1);	
}

void set_irq_handler(int pos, isr_t handler)
{
	ihandlers[pos]=handler;
}
void set_isr_handler(int pos, isr_t handler)
{
	ihandlers[pos]=handler;
}

void send_EOI(int num)
{
	 if(num >= 8)
	  outb(0xA0, 0x20);
	 outb(0x20, 0x20);
}

void *memset(void *s,int val,size_t size)
{
	char *a=(char *)s;
	while(size--)
	 *a++=val;
	return s;
}

void *memcpy(void *dest, void *src, size_t size)
{
	char *d=(char *)dest; char *s=(char *)src;
	while(size--)
	 *d++=*s++;
	return dest;
}

#ifdef _timer

static void timer_handler(regs_t regs)
{
	uint16 pos;
	
	pos=get_cursor_pos()/2;
	timer_ticks++;
	set_cursor_pos(time_cur_pos);
	print_long_(timer_ticks);
	set_cursor_pos(pos);
	
	if(regs.int_no==0)
	{}	
}

void init_timer(uint32 frequency)
{
	uint32 divisor; uint8 l,h;
	set_irq_handler(0,timer_handler);
	divisor=1193180/frequency;
	outb(0x43,0x36);	
	l=(uint8)(divisor&0xFF);
	h=(uint8)((divisor>>8)&0xFF);
	outb(0x40,l);
	outb(0x40,h);
}
#endif
