#ifndef typedefs_h
#define typedefs_h

#ifndef _nprimitive_vartypes  //typedefs for primitive data types
typedef unsigned char uchar;
typedef unsigned short ushort;
//typedef unsigned long size_t;
typedef unsigned char 	byte;
typedef unsigned short 	word;
typedef unsigned int 	dword;
typedef short    int 	bool;
typedef signed   char 	int8_t;
typedef unsigned char 	uint8_t;
typedef signed   short int 	int16_t;
typedef unsigned short int 	uint16_t;
typedef signed   int 	int32_t;
typedef unsigned int 	uint32_t;
typedef unsigned long long 	uint64_t;
typedef long long 	int64_t;
typedef int8_t 	int8;
typedef uint8_t 	uint8;
typedef int16_t 	int16;
typedef uint16_t 	uint16;
typedef int32_t 	int32;
typedef uint32_t 	uint32;
typedef int64_t 	int64;
typedef uint64_t 	uint64;
typedef unsigned char 	uchar_t;
typedef uint32_t 	wchar_t;
typedef uint32_t 	size_t;
typedef uint32_t 	addr_t;
typedef int32_t 	pid_t;
#endif

#ifndef _nstvartypes //typedefs from structs

struct gdt_entry_bits
{
	unsigned int limit_low:16;
	unsigned int base_low : 24;
     //attribute byte split into bitfields
	unsigned int accessed :1;
	unsigned int read_write :1; //readable for code, writable for data
	unsigned int conforming_expand_down :1; //conforming for code, expand down for data
	unsigned int code :1; //1 for code, 0 for data
	unsigned int always_1 :1; //should be 1 for everything but TSS and LDT
	unsigned int DPL :2; //priveledge level
	unsigned int present :1;
     //and now into granularity
	unsigned int limit_high :4;
	unsigned int available :1;
	unsigned int always_0 :1; //should always be 0
	unsigned int big :1; //32bit opcodes for code, dword stack for data
	unsigned int gran :1; //1 to use 4k page addressing, 0 for byte addressing
	unsigned int base_high :8;
} __packed; //or __attribute__((packed))

struct idt_entry_s{
   uint16 base_low;    // The lower 16 bits of the address to jump to when this interrupt occures: offset 0-15
   uint16 selector;      // Kernel segment selector in IDT
   uint8  zero;          // This must always be zero.
   uint8  flags;    //gate types, atttribute types etc.
   uint16 base_high;   // The upper 16 bits of the address to jump to. offset 16-31
} __attribute__((packed));

typedef struct idt_entry_s idt_entry_t;

struct idt_ptr_s_ {
    uint16 limit;
    uint32 base;     
} __attribute__((packed));
typedef struct idt_ptr_s_ idt_ptr_t;

struct gdt_entry_s
{
   uint16 limit_low;           // The lower 16 bits of the limit.
   uint16 base_low;            // The lower 16 bits of the base.
   uint8  base_middle;         // The next 8 bits of the base.
   uint8  access;              // Access flags, determine what ring this segment can be used in.
   uint8  granularity;         // eg: 4kb means 32 bit opcodes
   uint8  base_high;           // The last 8 bits of the base.
} __attribute__((packed));
typedef struct gdt_entry_s gdt_entry_t;

struct gdt_ptr_s_
{
   uint16 limit;               // The upper 16 bits of all selector limits.
   uint32 base;                // The address of the first gdt_entry_t struct.
}
 __attribute__((packed));
typedef struct gdt_ptr_s_ gdt_ptr_t; 

typedef struct kbd_status_s
{
	char kbd_buffer[32];
	char *kbuf_start,*kbuf_end,*kbuf;
	uint8 alt,shift,ctrl;
}kbd_status_t;
typedef struct registers_s
{
    unsigned int ds;
    unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax, unknown; //for popa
    unsigned int int_no, err_code;
    unsigned int eip, cs, eflags, useresp, ss;
} regs_t; /*
typedef struct
{
    uint32 ds;
    uint32 edi,esi,ebp,esp,ebx,edx,ecx,eax;   //pusha
    uint32 int_no, err_code;  //interrupt number and error code(if applicable)
    uint32 eip, cs, eflags, useresp, ss;  // pushed by the processor automatiocally
}regs_t;
*/
#endif

#endif
