; external references to the C kernel:
; idt_ptr
; gdt_ptr
; interrupt_handler

[BITS 32]
;======externally referred functions/ global functions=========================
global idt_set_1
global idt_set_2
global gdtflush
global Sum
global gdt_set_1
global gdt_set_2
global isr_h
global irq_h
;==============================================================================

;======external references=====================================================
extern interrupt_handler
extern isr_handler
extern irq1_keyb
extern timer_handler_
extern send_EOI

extern gdt_ptr
extern idt_ptr
;==============================================================================

idt_set_1:
     lidt [idt_ptr]

idt_set_2:
     mov eax,[esp+4]
     lidt [eax]
     ret

Sum:
     push    ebp	     ; create stack frame
     mov     ebp, esp
     mov     eax, [ebp+8]    ; grab the first argument
     mov     ecx, [ebp+12]   ; grab the second argument
     add     eax, ecx	     ; sum the arguments
     mov     ecx, [ebp+16]
     add     eax,ecx
     pop     ebp	     ; restore the base pointer
     ret


gdt_set_1:
    lgdt [gdt_ptr]
    mov ax, 0x10
    mov ds, ax
    mov es, ax
    mov fs, ax
    mov gs, ax
    mov ss, ax
    jmp 0x08:.flush2_
.flush2_:
    ret

gdt_set_2:
     mov eax,[esp+4]
     lgdt [eax]
     mov ax, 0x10
     mov ds, ax
     mov es, ax
     mov fs, ax
     mov gs, ax
     mov ss, ax
     jmp 0x08:.flush
.flush:
     ret

isr_h:
   pusha		    ; push edi,esi,ebp,esp,ebx,edx,ecx,eax

   mov ax, ds		    ; Lower 16-bits of eax = ds.
   push eax

   mov ax, 0x10  ;kernel data segment descriptor
   mov ds, ax
   mov es, ax
   mov fs, ax
   mov gs, ax
   mov ss, ax

   call isr_handler

   pop eax	  ;original data segment descriptor
   mov ds, ax
   mov es, ax
   mov fs, ax
   mov gs, ax
   mov ss, ax

   popa
   add esp, 8
   sti
   iret

irq_h:
   pusha		    ; push edi,esi,ebp,esp,ebx,edx,ecx,eax

   mov ax, ds		    ; Lower 16-bits of eax = ds.
   push eax

   mov ax, 0x10  ;kernel data segment descriptor
   mov ds, ax
   mov es, ax
   mov fs, ax
   mov gs, ax
   mov ss, ax

   call interrupt_handler

   pop eax	  ;original data segment descriptor
   mov ds, ax
   mov es, ax
   mov fs, ax
   mov gs, ax
   mov ss, ax

   popa
   add esp, 8
   sti
   iret


;======IRQS====================================================================

;======externally referred functions/ global functions=========================
global irq_0
global irq_1
global irq_2
global irq_3
global irq_4
global irq_5
global irq_6
global irq_7
global irq_8
global irq_9
global irq_10
global irq_11
global irq_12
global irq_13
global irq_14
global irq_15
;==============================================================================
extern irq1_keyb
extern timer_handler_
;==IRQ routines================================================================

irq_0:
  cli
  push byte 0 ;error
  push byte 0 ;interrupt number
  call irq_h

irq_1:
  cli
  push byte 0 ;error
  push byte 1 ;interrupt number
  call irq_h

irq_2:
  cli
  push byte 0 ;error
  push byte 2 ;interrupt number
  call irq_h

irq_3:
  cli
  push byte 0 ;error
  push byte 3 ;interrupt number
  call irq_h

irq_4:
  cli
  push byte 0 ;error
  push byte 4 ;interrupt number
  call irq_h

irq_5:
  cli
  push byte 0 ;error
  push byte 5 ;interrupt number
  call irq_h

irq_6:
  cli
  push byte 0 ;error
  push byte 6 ;interrupt number
  call irq_h

irq_7:
  cli
  push byte 0 ;error
  push byte 7 ;interrupt number
  call irq_h

irq_8:
  cli
  push byte 0 ;error
  push byte 8 ;interrupt number
  call irq_h

irq_9:
  cli
  push byte 0 ;error
  push byte 9 ;interrupt number
  call irq_h

irq_10:
  cli
  push byte 0  ;error
  push byte 10 ;interrupt number
  call irq_h

irq_11:
  cli
  push byte 0  ;error
  push byte 11 ;interrupt number
  call irq_h

irq_12:
  cli
  push byte 0  ;error
  push byte 12 ;interrupt number
  call irq_h

irq_13:
  cli
  push byte 0  ;error
  push byte 13 ;interrupt number
  call irq_h

irq_14:
  cli
  push byte 0  ;error
  push byte 14 ;interrupt number
  call irq_h

irq_15:
  cli
  push byte 0  ;error
  push byte 15 ;interrupt number
  call irq_h
;==============================================================================
;==============================================================================


;======ISRS====================================================================

;======externally referred functions/ global functions=========================
global _isr0
global _isr1
global _isr2
global _isr3
global _isr4
global _isr5
global _isr6
global _isr7
global _isr8
global _isr9
global _isr10
global _isr11
global _isr12
global _isr13
global _isr14
global _isr15
global _isr16
global _isr17
global _isr18
global _isr19
global _isr20
global _isr21
global _isr22
global _isr23
global _isr24
global _isr25
global _isr26
global _isr27
global _isr28
global _isr29
global _isr30
global _isr31
;==============================================================================
;======fault and exception handlers============================================
;  0: Divide By Zero Exception
_isr0:
    cli
    push byte 0
    push byte 0
    jmp isr_h

;  1: Debug Exception
_isr1:
    cli
    push byte 0
    push byte 1
    jmp isr_h

;  2: Non Maskable Interrupt Exception
_isr2:
    cli
    push byte 0
    push byte 2
    jmp isr_h

;  3: Int 3 Exception
_isr3:
    cli
    push byte 0
    push byte 3
    jmp isr_h

;  4: INTO Exception
_isr4:
    cli
    push byte 0
    push byte 4
    jmp isr_h

;  5: Out of Bounds Exception
_isr5:
    cli
    push byte 0
    push byte 5
    jmp isr_h

;  6: Invalid Opcode Exception
_isr6:
    cli
    push byte 0
    push byte 6
    jmp isr_h

;  7: Coprocessor Not Available Exception
_isr7:
    cli
    push byte 0
    push byte 7
    jmp isr_h

;  8: Double Fault Exception (With Error Code!)
_isr8:
    cli
    push byte 8
    jmp isr_h

;  9: Coprocessor Segment Overrun Exception
_isr9:
    cli
    push byte 0
    push byte 9
    jmp isr_h

; 10: Bad TSS Exception (With Error Code!)
_isr10:
    cli
    push byte 10
    jmp isr_h

; 11: Segment Not Present Exception (With Error Code!)
_isr11:
    cli
    push byte 11
    jmp isr_h

; 12: Stack Fault Exception (With Error Code!)
_isr12:
    cli
    push byte 12
    jmp isr_h

; 13: General Protection Fault Exception (With Error Code!)
_isr13:
    cli
    push byte 13
    jmp isr_h

; 14: Page Fault Exception (With Error Code!)
_isr14:
    cli
    push byte 14
    jmp isr_h

; 15: Reserved Exception
_isr15:
    cli
    push byte 0
    push byte 15
    jmp isr_h

; 16: Floating Point Exception
_isr16:
    cli
    push byte 0
    push byte 16
    jmp isr_h

; 17: Alignment Check Exception
_isr17:
    cli
    push byte 0
    push byte 17
    jmp isr_h

; 18: Machine Check Exception
_isr18:
    cli
    push byte 0
    push byte 18
    jmp isr_h

; 19: Reserved
_isr19:
    cli
    push byte 0
    push byte 19
    jmp isr_h

; 20: Reserved
_isr20:
    cli
    push byte 0
    push byte 20
    jmp isr_h

; 21: Reserved
_isr21:
    cli
    push byte 0
    push byte 21
    jmp isr_h

; 22: Reserved
_isr22:
    cli
    push byte 0
    push byte 22
    jmp isr_h

; 23: Reserved
_isr23:
    cli
    push byte 0
    push byte 23
    jmp isr_h

; 24: Reserved
_isr24:
    cli
    push byte 0
    push byte 24
    jmp isr_h

; 25: Reserved
_isr25:
    cli
    push byte 0
    push byte 25
    jmp isr_h

; 26: Reserved
_isr26:
    cli
    push byte 0
    push byte 26
    jmp isr_h

; 27: Reserved
_isr27:
    cli
    push byte 0
    push byte 27
    jmp isr_h

; 28: Reserved
_isr28:
    cli
    push byte 0
    push byte 28
    jmp isr_h

; 29: Reserved
_isr29:
    cli
    push byte 0
    push byte 29
    jmp isr_h

; 30: Reserved
_isr30:
    cli
    push byte 0
    push byte 30
    jmp isr_h

; 31: Reserved
_isr31:
    cli
    push byte 0
    push byte 31
    jmp isr_h

;==============================================================================
;==============================================================================